﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Jemgine.MapEditor
{
    [Tools.SelectableTool]
    public class TranslateTool : Tool
    {
        List<Operations.TransformItem> MultiSelectTransforms = new List<Operations.TransformItem>();
        Vector2 MouseDelta;
        bool Changes = false;

        public override System.Drawing.Image GetIcon() { return System.Drawing.Image.FromFile("ToolIcons/translate.bmp"); }


        public override void MouseDown(MouseInputState InputState, EditorState EditorState)
        {
            MultiSelectTransforms.Clear();
            var ItemsSelected = EditorState.GetEverythingSelected();
            foreach (var Item in ItemsSelected)
            {
                var SelectedObject = EditorState.GetSelectionObject(Item);
                var Translatable = SelectedObject as Data.Translatable;
                if (Translatable != null)
                {
                    var TransformationTag = Translatable.PrepareToTranslate();
                    if (TransformationTag is Data.VertexTranslateTag)
                        (TransformationTag as Data.VertexTranslateTag).Owner =
                            EditorState.GetMiddleObject(Item) as Data.HasVerticies;

                    MultiSelectTransforms.Add(new Operations.TransformItem
                    {
                        TransformationTag = TransformationTag,
                        Transformee = Translatable
                    });
                }
            }

            MouseDelta = Vector2.Zero;
            Changes = false;
        }

        public override void MouseUp(MouseInputState InputState, EditorState EditorState)
        {
            if (Changes && MultiSelectTransforms.Count != 0)
                EditorState.PerformOperation(new Operations.Translate(MultiSelectTransforms, MouseDelta,
                    (InputState.Modifiers & System.Windows.Forms.Keys.Alt) > 0));
            MultiSelectTransforms.Clear();
        }

        public override void MouseMove(MouseInputState InputState, EditorState EditorState)
        {
            MouseDelta += InputState.Delta;
            Changes = true;
            foreach (var Item in MultiSelectTransforms)
            {
                (Item.Transformee as Data.Translatable).ApplyTranslation(Item.TransformationTag, MouseDelta);
                if ((InputState.Modifiers & System.Windows.Forms.Keys.Alt) > 0)
                    (Item.Transformee as Data.Translatable).Snap(Operations.Snapper.GetSnapper(EditorState));
            }
        }
    }
}
