float4x4 World;
float4x4 View;
float4x4 Projection;

texture Texture;

sampler Sampler = sampler_state
{
    Texture = (Texture);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Point;
    
    AddressU = Clamp;
    AddressV = Clamp;
};


struct VertexShaderInput
{
	float2 Corner : POSITION0;
	float2 Position : POSITION1;
	float4 Color : COLOR0;
	float Scale : TEXCOORD0;
	float Angle : TEXCOORD1;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float4 Color : COLOR0;
    float2 TextureCoordinate : TEXCOORD0;
};

VertexShaderOutput ParticleVertexShader(VertexShaderInput input)
{
	VertexShaderOutput Output;

	float c = cos(input.Angle);
    float s = sin(input.Angle);
    
    float2 cornerPosition = input.Corner * input.Scale;
	cornerPosition = mul(cornerPosition, float2x2(c, -s, s, c));
	cornerPosition = input.Position + cornerPosition;

	float4 worldPosition = mul(float4(cornerPosition, 0, 1), World);
    float4 viewPosition = mul(worldPosition, View);
    float4 projPosition = mul(viewPosition, Projection);
	
	Output.Position = projPosition;
	Output.Color = input.Color;
	Output.TextureCoordinate = (input.Corner / 2.0) + float2(0.5, 0.5);

	return Output;
}

float4 ParticlePixelShader(VertexShaderOutput input) : COLOR0
{
    return tex2D(Sampler, input.TextureCoordinate) * input.Color;
}

technique prtBlend
{
    pass P0
    {
		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
		ZEnable = false;
		ZWriteEnable = false;
		ZFunc = LessEqual;

        VertexShader = compile vs_2_0 ParticleVertexShader();
        PixelShader = compile ps_2_0 ParticlePixelShader();
    }
}

technique prtAdd
{
    pass P0
    {
		AlphaBlendEnable = true;
		BlendOp = Add;
		SrcBlend = One;
		DestBlend = One;
		ZEnable = false;
		ZWriteEnable = false;
		ZFunc = LessEqual;

        VertexShader = compile vs_2_0 ParticleVertexShader();
        PixelShader = compile ps_2_0 ParticlePixelShader();
    }
}

