﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using System.ComponentModel;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

namespace Jemgine.Data
{
        [Jemgine.Engine.Category("Primitive"), Jemgine.Engine.Component]
	public class Spline : EntityElement, EditorDrawable, TexturedObject, 
        Translatable, Rotatable, Scalable, ReferenceTarget, HasVerticies,
        Engine.Renderer.IDrawable, Engine.Renderer.ICastShadow
	{
        public enum TextureMappingStyle
        {
            Horizontal,
            HorizontalFlipped,
            Vertical,
            VerticalFlipped,
        }

        public int BoneID { get; set; }
        [ContentSerializerIgnore]
        public Bone BoneReference { get; set; }

        private TextureMappingStyle _mappingStyle = TextureMappingStyle.Horizontal;
        public TextureMappingStyle MappingStyle { get { return _mappingStyle; } 
            set { _mappingStyle = value; ClearDecomposedCache(); } }

        public Engine.GeometryType Type { get; set; }
        [ContentSerializer(Optional = true)]
        public Engine.Renderer.ShadowType ShadowType { get; set; }
       
		[Microsoft.Xna.Framework.Content.ContentSerializerIgnore]
        public int ReferenceID { get; set; }

        private float _width = 1.0f;
        public float Width { get { return _width; } set { _width = value; ClearDecomposedCache(); } }

        private int _segments = 10;
        public int Segments { get { return _segments; } set { _segments = value; ClearDecomposedCache(); } }

        private Vector2 _textureScale = Vector2.One;
        public Vector2 TextureScale { get { return _textureScale; } set { _textureScale = value; ClearDecomposedCache(); } }

        Vector2 ReferenceTarget.GetWorldPosition()
        {
            if (Owner == null) return Vector2.Zero;
            else return Owner.Position + Data[1].Position;
        }

        [Microsoft.Xna.Framework.Content.ContentSerializerIgnore]
        public EntityRecord Owner { get; private set; }

        public void SetOwner(EntityRecord E)
        {
            Owner = E;
        }

		public Spline(Spline p)
		{
            for (int i = 0; i < 3; ++i)
                Data[i] = new Vertex(p.Data[i].Position);
			Texture = p.Texture;
            Width = p.Width;
            Segments = p.Segments;
            TextureScale = p.TextureScale;
            _mappingStyle = p._mappingStyle;
            Type = p.Type;
            BoneID = p.BoneID;
            
            Decomposed = null;
		}

		public Spline()
		{
            for (int i = 0; i < 3; ++i)
                Data[i] = new Vertex();
            Width = 1;
            Segments = 10;
            TextureScale = Vector2.One;
            Type = Engine.GeometryType.Both;
            
            Decomposed = null;
		}

        [Microsoft.Xna.Framework.Content.ContentSerializerIgnore]
        public VertexPositionTexture[] Decomposed = null;

        public void UpdateDecomposedCache()
        {
            if (Segments > 64) Segments = 64;
            float Step = 1.0f / (float)Segments;
            float Place = 0.0f;
            int TotalPoints = (Segments + 1) * 2;
            Decomposed = new VertexPositionTexture[TotalPoints];

            for (int i = 0; i <= Segments; ++i)
            {
                Vector2 Position = Jemgine.Engine.Bezier.Point(this[0].Position, this[1].Position, this[2].Position, Place);
                Vector2 Tangent = Jemgine.Engine.Bezier.Tangent(this[0].Position, this[1].Position, this[2].Position, Place);

                Tangent = Jemgine.Math.Vector.Perpendicular(Tangent);

                Decomposed[i * 2].Position = new Vector3(Position + (Tangent * Width / 2.0f), 0);
                Decomposed[(i * 2) + 1].Position = new Vector3(Position - (Tangent * Width / 2.0f), 0);

                switch (MappingStyle)
                {
                    case TextureMappingStyle.Horizontal:
                        Decomposed[i * 2].TextureCoordinate = new Vector2(Place, 0) * TextureScale;
                        Decomposed[(i * 2) + 1].TextureCoordinate = new Vector2(Place, 1) * TextureScale;
                        break;
                    case TextureMappingStyle.HorizontalFlipped:
                        Decomposed[i * 2].TextureCoordinate = new Vector2(1.0f - Place, 0) * TextureScale;
                        Decomposed[(i * 2) + 1].TextureCoordinate = new Vector2(1.0f - Place, 1) * TextureScale;
                        break;
                    case TextureMappingStyle.Vertical:
                        Decomposed[i * 2].TextureCoordinate = new Vector2(0, Place) * TextureScale;
                        Decomposed[(i * 2) + 1].TextureCoordinate = new Vector2(1, Place) * TextureScale;
                        break;
                    case TextureMappingStyle.VerticalFlipped:
                        Decomposed[i * 2].TextureCoordinate = new Vector2(0, 1.0f - Place) * TextureScale;
                        Decomposed[(i * 2) + 1].TextureCoordinate = new Vector2(1, 1.0f - Place) * TextureScale;
                        break;
                }

                Place += Step;
            }
        }

        public void ClearDecomposedCache()
        {
            Decomposed = null;
        }

        public Vertex[] Data = new Vertex[3];

        [System.ComponentModel.Editor(typeof(TextureEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public String Texture { get; set; }
        
		public int Count { get { return 3; } }
		public Vertex this[int _index] 
		{ 
			get { return Data[_index]; }
            set { Data[_index] = value; ClearDecomposedCache(); }
		}
        public IVertex GetVertex(int Index) { return this[Index] as IVertex; }

        public Math.AABB GetAABB()
        {
            float MinDimension = System.Math.Max(Width, 1);

            Vector2 Min = this[0].Position;
            Vector2 Max = this[0].Position;

            for (int i = 1; i < 3; ++i)
            {
                if (this[i].Position.Y > Max.Y) Max.Y = this[i].Position.Y;
                if (this[i].Position.Y < Min.Y) Min.Y = this[i].Position.Y;
                if (this[i].Position.X > Max.X) Max.X = this[i].Position.X;
                if (this[i].Position.X < Min.X) Min.X = this[i].Position.X;
            }

            Math.AABB Result = new Math.AABB(Min.X, Min.Y, Max.X - Min.X, Max.Y - Min.Y);

                Result.X -= MinDimension / 2;
                Result.Width += MinDimension;
                Result.Y -= MinDimension / 2;
                Result.Height += MinDimension;

            return Result;
        }

		public bool PointInside(Vector2 point)
		{
            var AABB = GetAABB();
            return Math.AABB.Inside(ref AABB, ref point);
		}

        public Jemgine.Engine.Selection.Types SelectionType()
        {
            return Jemgine.Engine.Selection.Types.EntityElement;
        }

        public EntityElement Clone()
        {
            return new Spline(this);
        }

        public void Initialize(Data.EntityRecord Self, Microsoft.Xna.Framework.Content.ContentManager Content)
        {
            UpdateDecomposedCache();
        }

        public void Destroy()
        { }


		#region Translatable Members

        object Translatable.PrepareToTranslate()
        {
            List<Vector2> Positions = new List<Vector2>();
            foreach (var Vertex in Data) Positions.Add(Vertex.Position);
            return Positions;
        }

        void Translatable.ApplyTranslation(object Tag, Vector2 MouseDelta)
        {
            List<Vector2> Positions = Tag as List<Vector2>;
            for (int i = 0; i < Count; ++i)
                Data[i].Position = Positions[i] + MouseDelta;
            ClearDecomposedCache();
        }

        void Translatable.Snap(Snapper Snapper)
        {
            foreach (var Vertex in Data)
                Vertex.Position = Snapper.SnapPosition(Vertex.Position);
        }

        #endregion

        #region Rotatable Members

        object Rotatable.PrepareToRotate()
        {
            List<Vector2> Positions = new List<Vector2>();
            foreach (var Vertex in Data) Positions.Add(Vertex.Position);
            return Positions;
        }

        void Rotatable.ApplyRotation(object Tag, float Rotation)
        {
            List<Vector2> Positions = Tag as List<Vector2>;
            Vector2 Center = Vector2.Zero;
            foreach (var V in Positions) Center += V;
            Center /= Positions.Count;

            Matrix M = Matrix.CreateRotationZ(Rotation);

            for (int I = 0; I < Positions.Count; ++I)
                Data[I].Position = Center + Vector2.Transform(Positions[I] - Center, M);

            ClearDecomposedCache();
        }

        void Rotatable.Snap(Snapper Snapper)
        {
            ( this as Translatable ).Snap(Snapper);
        }

        #endregion

        #region EditorDrawable Members

        public void Draw(int ID, EditorDrawData Context, SelectionType Selected)
        {
            if (Decomposed == null) UpdateDecomposedCache();

            Context.Renderer.Color = EditorDrawData.DrawColor(Selected);

            Texture2D _Texture = null;
            if (Selected == Jemgine.Data.SelectionType.NotSelected)
                _Texture = Context.GetTexture(Texture);
            Context.Renderer.DrawSpline(this, _Texture);

            if (Selected == Jemgine.Data.SelectionType.Selected)
                foreach (var point in Data)
                    Context.Renderer.DrawPoint(point.Position);
            if (Selected == Jemgine.Data.SelectionType.WorkingSet || Selected == Jemgine.Data.SelectionType.Selected)
            {
                var AABB = GetAABB();
                Context.Renderer.DrawBox(AABB.Min, AABB.Max, 0.2f);
            }
        }

        #endregion


        #region TexturedObject Members


        public void SetTexture(String texture, Texture2D _tex)
        {
            this.Texture = texture;
        }

        #endregion

        #region Scalable Members

        object Scalable.PrepareToScale()
        {
            List<Vector2> Positions = new List<Vector2>();
            foreach (var Vertex in Data) Positions.Add(Vertex.Position);
            return Positions;
        }

        void Scalable.ApplyScale(object Tag, float Delta)
        {
            List<Vector2> Positions = Tag as List<Vector2>;
            Vector2 Center = Vector2.Zero;
            foreach (var V in Positions) Center += V;
            Center /= Positions.Count;

            float MinY = Positions[0].Y;
            float MaxY = Positions[0].Y;
            foreach (var V in Positions)
            {
                if (V.Y > MaxY) MaxY = V.Y;
                if (V.Y < MinY) MinY = V.Y;
            }

            float Range = MaxY - MinY;
            float NewRange = Range + Delta;

            Matrix M = Matrix.CreateScale(NewRange / Range);

            for (int I = 0; I < Positions.Count; ++I)
                Data[I].Position = Center + Vector2.Transform(Positions[I] - Center, M);

            ClearDecomposedCache();        
        }

        void Scalable.Snap(Snapper Snapper)
        {
            ( this as Translatable ).Snap(Snapper);
        }

        #endregion

        void Engine.Renderer.IDrawable.Draw(Engine.Renderer.RenderEffect Effect, Engine.Renderer.VertexPusher Pusher, Engine.EpisodeContentManager Content)
        {
            if ((Type & Engine.GeometryType.Draw) != Engine.GeometryType.Draw) return;

            var Tex = Content.GetTexture(Texture);
            if (Tex == null) Tex = Effect.LineTexture;
            Effect.Texture = Tex;
            if (Decomposed == null) UpdateDecomposedCache();

            Pusher.DrawTriangleStrip(this.Decomposed);
        }

        void Engine.Renderer.ICastShadow.DrawShadow(Engine.Renderer.RenderEffect Effect,
            Vector2 LightPosition, Matrix Transform, Engine.Renderer.VertexPusher Pusher)
        {
            if (ShadowType == Engine.Renderer.ShadowType.None) return;
            //if ((Type & Engine.GeometryType.Draw) != Engine.GeometryType.Draw) return;

            if (Segments > 64) Segments = 64;
            float Step = 1.0f / (float)Segments;
            float Place = 0.0f;
            Vector2 LastPos = Data[0].Position;

            for (int i = 0; i <= Segments; ++i)
            {
                Vector2 Position = Jemgine.Engine.Bezier.Point(Data[0].Position, Data[1].Position, Data[2].Position, Place);
                Place += Step;
                Line.DrawSegmentShadow(LastPos, Position, LightPosition, Transform, Pusher, false, Effect);
                LastPos = Position;
            }
        }
    }
}
