﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;

namespace Antioch.Base
{

    [XmlRoot("dictionary")]
    public class SerializableDictionary<TKey, TValue>
        : Dictionary<TKey, TValue>, IXmlSerializable
    {
        public SerializableDictionary() { }

        public SerializableDictionary(SerializableDictionary<TKey, TValue> Other)
        {
            foreach (var Item in Other) this.Add(Item.Key, Item.Value);
        }

        public System.Xml.Schema.XmlSchema GetSchema() { return null; }

        public void ReadXml(System.Xml.XmlReader reader)
        {
XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
            XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));

            bool wasEmpty = reader.IsEmptyElement;
            reader.Read();

            if (wasEmpty) return;

            while (reader.NodeType != System.Xml.XmlNodeType.EndElement)
            {
reader.ReadStartElement("item");
                reader.ReadStartElement("key");
                TKey key = (TKey)keySerializer.Deserialize(reader);
                reader.ReadEndElement();

                reader.ReadStartElement("value");
                TValue value = (TValue)valueSerializer.Deserialize(reader);
                reader.ReadEndElement();

                this.Add(key, value);
                reader.ReadEndElement();
                reader.MoveToContent();
            }
            reader.ReadEndElement();
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
            XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));

            foreach (TKey key in this.Keys)
            {
                writer.WriteStartElement("item");
                writer.WriteStartElement("key");
                keySerializer.Serialize(writer, key);
                writer.WriteEndElement();
                writer.WriteStartElement("value");
                TValue value = this[key];
                valueSerializer.Serialize(writer, value);
                writer.WriteEndElement();
                writer.WriteEndElement();
            }
        }
    }

    public class SaveableState
    {
        public String MapName = "MainMenu";
        public String NodeToFire = "SpawnPlayer";

        public int PlayerHealth = 100;
        public int MaxPlayerHealth = 100;

        public SerializableDictionary<String, int> Variables = new SerializableDictionary<string, int>();

        public SaveableState() { }

        public SaveableState(SaveableState Other)
        {
            CopyFrom(Other);
        }

        public void CopyFrom(SaveableState Other)
        {
            this.MapName = Other.MapName;
            this.NodeToFire = Other.NodeToFire;
            this.PlayerHealth = Other.PlayerHealth;
            this.MaxPlayerHealth = Other.MaxPlayerHealth;
            this.Variables = new SerializableDictionary<string, int>(Other.Variables);
        }


    }
}
